@@ -169,10 +169,13 @@
 #ifndef _FFTN_C
 #define _FFTN_C
 /* we use CPP to re-include this same file for double/float cases */
+
 #if !defined (lint) && !defined (__FILE__)
-Error: your compiler is sick!  define __FILE__ yourself (a string)
-eg, something like -D__FILE__=\"fftn.c\"
+#error Your compiler is sick!
+/* define __FILE__ yourself (a string) eg, something like
+D__FILE__=\"fftn.c\" */
 #endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -351,7 +354,7 @@ FFTN (int ndim,
       REAL Re [],
       REAL Im [],
       int iSign,
-      double scaling)
+      REAL scaling)
 {
    size_t nTotal;
    int maxFactors, maxPerm;
@@ -449,6 +452,7 @@ FFTN (int ndim,
 	if (iSign < 0) iSign = -iSign;
 	if (scaling < 0.0)
 	  scaling = (scaling < -1.0) ? sqrt (nTotal) : nTotal;
+
 	scaling = 1.0 / scaling;	/* multiply is often faster */
 	for (i = 0; i < nTotal; i += iSign)
 	  {
